package me.unurled.srcore;

import static me.unurled.srcore.utils.LoggerUtils.error;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/** The Managers class is a class that is used to manage managers. */
public class Managers {

  private final List<Manager> managerList;

  /** Create a new manager */
  protected Managers() {
    managerList = new ArrayList<>();
  }

  /**
   * Register a manager
   *
   * @param clazz The class of the manager
   */
  public void register(@NotNull List<Class<? extends Manager>> clazz) {
    int i = 0;
    for (Class<? extends Manager> c : clazz) {
      Bukkit.getScheduler()
          .runTaskLater(
              SRCore.getInstance().getPlugin(),
              () -> {
                try {
                  c.getDeclaredConstructor().newInstance();
                } catch (InstantiationException
                    | IllegalAccessException
                    | InvocationTargetException
                    | NoSuchMethodException e) {
                  error("Failed to register manager: " + c.getSimpleName());
                }
              },
              10L * i);
      i++;
    }
  }

  /**
   * Add a manager
   *
   * @param manager The manager
   */
  public void addManager(Manager manager) {
    managerList.add(manager);
  }

  /** Unload all managers */
  public void unload() {
    for (Manager manager : managerList) {
      manager.unload();
    }
  }

  /**
   * Get a manager
   *
   * @param clazz The class of the manager
   * @return The manager
   */
  @Nullable
  public Manager getManager(Class<? extends Manager> clazz) {
    for (Manager manager : managerList) {
      if (manager.getClass().equals(clazz)) {
        return manager;
      }
    }
    return null;
  }
}
