package me.unurled.srcore.api;

import me.unurled.srcore.SRCore;
import org.bukkit.Bukkit;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

/** The Manager class is a class that is used to manage data and events. */
public class Manager implements Listener {

  /** Create a new manager */
  public Manager() {
    SRCore.getInstance().getManagers().addManager(this);
    Bukkit.getScheduler()
        .runTaskAsynchronously(
            SRCore.getInstance().getPlugin(),
            () -> {
              load();
              Bukkit.getPluginManager().registerEvents(this, SRCore.getInstance().getPlugin());
            });
  }

  /**
   * Get an instance of a manager
   *
   * @param clazz The class of the manager
   * @return The instance of the manager
   * @param <T> The type of the manager
   */
  public static <T extends Manager> T getInstance(@NotNull Class<T> clazz) {
    return clazz.cast(SRCore.getInstance().getManagers().getManager(clazz));
  }

  /** Load the manager */
  public void load() {
    loadData();
  }

  /** Unload the manager */
  public void unload() {
    saveData();
  }

  /** Save the data */
  public void saveData() {
    /* method empty, so it isn't required by the extended child to implement it */
  }

  /** Load the data */
  public void loadData() {
    /* method empty, so it isn't required by the extended child to implement it */
  }
}
