package me.unurled.srcore.utils;

import static me.unurled.srcore.utils.ReflectionUtils.*;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

/** The ReflectionRegistry class is a class that is used to store reflection classes and methods. */
public class ReflectionRegistry {
  /** The PLUGIN_CLASS_LOADER_CLASS class is a class that is used to load plugins in Bukkit. */
  public static final Class<?> PLUGIN_CLASS_LOADER_CLASS =
      ReflectionUtils.getClass("org.bukkit.plugin.java.PluginClassLoader");

  /** The PaperPluginClassLoader class is a class that is used to load plugins in PaperMC. */
  public static final Class<?> PAPER_PLUGIN_CLASS_LOADER_CLASS =
      getClassOrNull("io.papermc.paper.plugin.entrypoint.classloader.PaperPluginClassLoader");

  /**
   * The getLoadedJavaPlugin method is a method that is used to get the loaded JavaPlugin from the
   */
  public static final Method PAPER_PLUGIN_CLASS_LOADER_GET_LOADED_JAVA_PLUGIN_METHOD;

  /**
   * The PLUGIN_CLASS_LOADER_PLUGIN_FIELD field is a field that is used to get the plugin from the
   */
  public static final Field PLUGIN_CLASS_LOADER_PLUGIN_FIELD =
      getField(PLUGIN_CLASS_LOADER_CLASS, true, "plugin");

  static {
    if (PAPER_PLUGIN_CLASS_LOADER_CLASS != null) {
      Method getPlugin =
          getMethodOrNull(PAPER_PLUGIN_CLASS_LOADER_CLASS, false, "getLoadedJavaPlugin");
      if (getPlugin == null)
        getPlugin = getMethodOrNull(PAPER_PLUGIN_CLASS_LOADER_CLASS, false, "getPlugin");
      PAPER_PLUGIN_CLASS_LOADER_GET_LOADED_JAVA_PLUGIN_METHOD = getPlugin;
    } else {
      PAPER_PLUGIN_CLASS_LOADER_GET_LOADED_JAVA_PLUGIN_METHOD = null;
    }
  }

  private ReflectionRegistry() {}
}
